#version 460 core

in vec3 position;
in vec3 normal;
in vec3 tangent;
in vec3 bitangent;
in vec2 uv;

in mat3 tbn;

out vec4 o_colour;

// TODO Merge AO, Metallic, and Roughness into single texture for performance. 
uniform sampler2D albedo_tex;
uniform sampler2D ambient_occlusion_tex;
uniform sampler2D metallic_tex; 
uniform sampler2D normal_tex;
uniform sampler2D roughness_tex;

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};
uniform Light light;
uniform vec3 camPos;

vec3 get_normal_from_texture() {
    vec3 rgb = texture(normal_tex, uv).rgb;
    return normalize(rgb * 2.0 - 1.0);
}

void main() {
    vec3 texNormal = normalize(tbn * get_normal_from_texture()); 
    // vec3 texColour = pow(texture(albedo_tex, uv).rgb, vec3(1.0 / 2.2)); // Gamma correction
    vec3 texColour = texture(albedo_tex, uv).rgb;
    vec3 texAO = texture(ambient_occlusion_tex, uv).rgb;
    vec3 texMetallic = texture(metallic_tex, uv).rgb;
    vec3 texRoughness = texture(roughness_tex, uv).rgb;

    vec3 lightDir = normalize(light.position - position);

    vec3 ambient = light.ambient;

    float diff = max(dot(lightDir, texNormal), 0.0);
    vec3 diffuse = light.diffuse * diff;

    // Not completely sure if my specular calculations are correct, but the rest should be
    vec3 materialSpec = mix(vec3(0.04), texColour, texMetallic);
    vec3 reflectDir = reflect(-lightDir, texNormal);
    vec3 viewDir = normalize(camPos - position);
    float shininess = max(2 / pow(texRoughness.r, 4) - 2, 0.0);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
    vec3 specular = light.specular * spec * materialSpec;

    o_colour = vec4((texColour * (ambient + diffuse)) * texAO + specular * texAO, 1);
    //o_colour = vec4(specular, 1);
}