#version 460 core

in vec3 position;
in vec3 normal;
in vec4 clip_space;
in vec2 tex_coords;

out vec4 o_colour;

const float PI = 3.14159;

uniform sampler2D cloudTex;

uniform mat4 inv_model;
uniform vec3 obj_pos;
uniform vec3 colour;
uniform vec3 sky_colour;
uniform vec3 cam_pos;
uniform float ct;
uniform int march_steps = 4;
uniform float cloud_scale;
uniform float cloud_speed;
uniform float hg_g = 0.85;

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};
uniform Light light;

vec2 ray_box(vec3 o, vec3 d) {

    vec3 m = 1.0 / d;
    vec3 n = m * o;
    vec3 k = abs(m);
    vec3 t1 = -n - k;
    vec3 t2 = -n + k;
    float tN = max(max(t1.x, t1.y), t1.z);
    float tF = min(min(t2.x, t2.y), t2.z);
    if (tN > tF || tF < 0.0) return vec2(-1.0);
    return vec2(tN, tF);
}

// texture-based noise from https://www.shadertoy.com/view/4sfGzS
float noise(vec3 x) {
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f * f * (3.0 - 2.0 * f);
    vec2 uv = (p.xy + vec2(37.0, -17.0) * p.z) + f.xy;
    vec2 rg = textureLod(cloudTex, (uv + 0.5) / 256.0, 0.0).yx;
    return mix(rg.x, rg.y, f.z);
}

float fbm4(vec3 p) {    
    vec3 q = cloud_scale * (p + cloud_speed * ct * vec3(0.3, -1.7, 0.8));
    float f = 0.5 * noise(q); q = q * 2.0;
    f += 0.25 * noise(q); q = q * 2.0;
    f += 0.125 * noise(q); q = q * 2.0;
    f += 0.0625 * noise(q);    
    // returns value from 0 to 1
    return f / 0.9375;
}

float hg(float a) {
    float g2 = hg_g * hg_g;
    return (1 - g2) / (4 * PI * pow(1 + g2 - 2 * hg_g * a, 1.5));
}

float smin(float a, float b, float k) {
    float h = a - b;
    return 0.5 * ((a + b) - sqrt(h * h + k));
}

void main() {
    vec3 res_colour = colour;
    vec4 res_colour_alpha = vec4(colour, 1.0);
    vec3 cam_dir = normalize(position - cam_pos);
    vec3 light_dir = normalize(light.position - position);

    float cos_angle = dot(cam_dir, light_dir);
    float hg_factor = hg(cos_angle);

    res_colour_alpha = vec4(texture(cloudTex, tex_coords + cloud_speed * ct * vec2(0.3, -1.7)).r) * -smin(-0.5, -hg_factor, 0.01);
    
    o_colour = res_colour_alpha;
}