#version 460 core
#define MAX_BONES 15

layout (location = 0) in vec3 a_pos;
layout (location = 1) in vec3 a_normal;
layout (location = 2) in ivec4 a_bone_ids;
layout (location = 3) in vec4 a_bone_weights;

out vec3 position;
flat out vec3 normal;
// flat out ivec4 bone_ids;
// out vec4 bone_weights;

uniform mat4 vp;
uniform mat4 model;
uniform mat3 tinv_model;

uniform mat4 final_bone_matrices[MAX_BONES];
uniform bool has_bones;

mat3 tinv(mat4 matrix) {
    return transpose(inverse(mat3(matrix)));
}

void main() {
    if (has_bones) {
        vec4 final_pos = vec4(0);
        vec3 final_norm = vec3(0);
        for (int i = 0; i < 4; i++) {
            if (a_bone_ids[i] == -1) {
                continue;
            }
            final_pos += a_bone_weights[i] * final_bone_matrices[a_bone_ids[i]] * vec4(a_pos, 1.0);
            // possible optimisation: compute transpose-inverse on cpu first
            final_norm += a_bone_weights[i] * tinv(final_bone_matrices[a_bone_ids[i]]) * a_normal;
        }
        position = vec3(model * final_pos);
        gl_Position = vp * vec4(position, 1.0);
        normal = normalize(tinv_model * final_norm);
    } else {
        position = vec3(model * vec4(a_pos, 1.0));
        gl_Position = vp * vec4(position, 1.0);
        normal = normalize(tinv_model * a_normal);
    }
}