#version 460 core

layout (location = 0) in vec3 a_pos;
layout (location = 1) in vec3 a_normal;

out vec3 position;
out vec3 normal;
out float ocean_ht;

uniform vec4 clip_plane;
uniform mat4 vp;
uniform mat4 model;
uniform float ct;

// compute the height of the wave at that point in time (estimated, based on params in ocean.vert)
const vec4 wave1 = vec4(1, 1, 0.1, 41);
const vec4 wave2 = vec4(1, 0, 0.05, 21);
const vec4 wave3 = vec4(0, -1, 0.05, 17);
const float PI = 3.14159;

vec3 gwave_i(vec4 params, vec3 p) {
    float steepness = params.z;
    float wavelength = params.w;
    vec2 d = normalize(params.xy);
    float k = 2 * PI / wavelength;
    float c = sqrt(9.8 / k);
    float f = k * (dot(d, p.xz) - c * ct);
    float a = steepness / k;

    return vec3(d.x * a * cos(f), a * sin(f), d.y * a * cos(f));
}

vec3 gwave(vec3 ip) {
    vec3 p = ip;
    p.y = 0;
    p += gwave_i(wave1, ip);
    p += gwave_i(wave2, ip);
    p += gwave_i(wave3, ip);
    return p;
}

// given a point p, find the height of the ocean at that pt
float ocean_height(vec3 p) {
    // find where this point ends up
    vec3 tp = p;
    vec3 np = gwave(tp);
    // step backwards by displacement
    vec2 dis = np.xz - tp.xz;
    tp.xz = p.xz - dis;
    np = gwave(tp);
    // step backwards
    dis = np.xz - tp.xz;
    tp.xz = p.xz - dis;
    np = gwave(tp);

    return np.y;
}


void main() {
    gl_Position = vp * model * vec4(a_pos, 1.0);
    position = vec3(model * vec4(a_pos, 1.0));
    normal = a_normal;
    ocean_ht = ocean_height(a_pos);
    gl_ClipDistance[0] = dot(vec4(position, 1.0), clip_plane);
}