#version 460 core

layout (location = 0) in vec3 a_pos;

const float PI = 3.14159;

out vec3 position;
out vec3 normal;
out vec4 clip_space;
out vec2 tex_coords;

uniform mat4 vp;
uniform mat4 model;
uniform float dudv_tiling;
uniform float ct;
uniform vec3 cam_pos;

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};
uniform Light light;

const vec4 wave1 = vec4(1, 1, 0.1, 41);
const vec4 wave2 = vec4(1, 0, 0.05, 21);
const vec4 wave3 = vec4(0, -1, 0.05, 17);

vec3 gwave(vec4 params, vec3 p, inout vec3 tangent, inout vec3 binormal) {
    float steepness = params.z;
    float wavelength = params.w;
    vec2 d = normalize(params.xy);
    float k = 2 * PI / wavelength;
    float c = sqrt(9.8 / k);
    float f = k * (dot(d, p.xz) - c * ct);
    float a = steepness / k;

    tangent += vec3(-d.x * d.x * steepness * sin(f), d.x * steepness * cos(f), -d.x * d.y * steepness * sin(f));
    binormal += vec3(-d.x * d.y * steepness * sin(f), d.y * steepness * cos(f), -d.y * d.y * steepness * sin(f));

    return vec3(d.x * a * cos(f), a * sin(f), d.y * a * cos(f));
}

void main() {
    vec3 p = a_pos;
    vec3 tangent = vec3(1, 0, 0);
    vec3 binormal = vec3(0, 0, 1);
    p += gwave(wave1, a_pos, tangent, binormal);
    p += gwave(wave2, a_pos, tangent, binormal);
    p += gwave(wave3, a_pos, tangent, binormal);
    normal = normalize(cross(binormal, tangent));

    clip_space = vp * model * vec4(p, 1.0);
    position = vec3(model * vec4(p, 1.0));
    gl_Position = clip_space;
    tex_coords = dudv_tiling * (vec2(p.x, p.z) * 0.5 + 0.5);
}